﻿using System;
using Microsoft.Extensions.Logging;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectGasolineTanker.Generic;
using ProjectGasolineTanker.Drawings;
using ProjectGasolineTanker.MovementStratg;

using ProjectGasolineTanker.Exceptions;
using ProjectGasolineTanker;
using System.Xml.Linq;
using ProjectGasolineTanker.Generics;

namespace ProjectGasolineTanker
{
    public partial class FormTruckCollection : Form
    {
        // Набор объектов
        private readonly TruckGenericStorage _storage;
        // Логер
        private readonly ILogger _logger;

        public FormTruckCollection(ILogger<FormTruckCollection> logger)
        {
            InitializeComponent();
            _storage = new TruckGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
            _logger = logger;
        }

        
        private void buttonSortByType_Click(object sender, EventArgs e) => CompareTruck(new TruckCompareByType());

        private void Sort_Color_button_Click(object sender, EventArgs e) => CompareTruck(new TruckCompareByColor());

        // Сортировка по сравнителю
        private void CompareTruck(IComparer<DrawingTruck?> comparer)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            obj.Sort(comparer);
            pictureBoxCollection.Image = obj.ShowTruck();
        }

        // заполнение лист бокс
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;
            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i].Name);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 && index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }

        }

        // добавить набор в коллекцию
        private void buttonAddStorage_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
            _logger.LogInformation($"Добавлен набор: {textBoxStorageName.Text}");
        }

        // выбрать набор
        private void listBoxObjects_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image = _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowTruck();
        }

        // удалить набор
        private void buttonDeleteStorage_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            string name = listBoxStorages.SelectedItem.ToString() ?? string.Empty;
            if (MessageBox.Show($"Удалить объект {listBoxStorages.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(name);
                ReloadObjects();
                _logger.LogInformation($"Удалён набор: {name}");
            }
        }


        private void buttonAddTruck_Click(object sender, EventArgs e)
        {
                
            var formTruckConfig = new FormTruckConfig();      
            formTruckConfig.AddEvent(AddTruck);
            formTruckConfig.Show();
        }

        private void AddTruck(DrawingTruck selectedTruck)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                MessageBox.Show("Не выбран набор");
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            selectedTruck.ChangePictureSize(Width, Height);
            try
            {
                if (obj + selectedTruck != -1)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = obj.ShowTruck();
                    _logger.LogInformation($"Добавлен объект: {selectedTruck.EntityTruck.BodyColor}");
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
            catch (StorageOverflowException ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogWarning(ex.Message);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogWarning(ex.Message);
            }
        }


        private void buttonDeleteTruck_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            try
            {
                int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
                if (obj - pos)
                {
                    MessageBox.Show("Объект удален");
                    pictureBoxCollection.Image = obj.ShowTruck();
                    _logger.LogInformation($"Удалён объект по позиции : {pos}");
                }
                else
                {
                    MessageBox.Show("Не удалось удалить объект");
                }
            }
            catch (FormatException ex)
            {
                MessageBox.Show("Неверный формат ввода");
                _logger.LogWarning("Неверный формат ввода");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogWarning(ex.Message);
            }
        }

        // обновить рисунок по набору
        private void buttonUpdate_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowTruck();
        }

        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Файл сохранён по пути: {saveFileDialog.FileName}");
                }
                catch (InvalidOperationException ex)
                {
                    MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning(ex.Message);
                }
            }
        }

        // Обработка нажатия "Загрузка"
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Файл загружен по пути: {openFileDialog.FileName}");
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning(ex.Message);
                }
            }
            ReloadObjects();
        }


    }
}
